/* This file is part of the wvWare 2 project
   Copyright (C) 2001-2003 Werner Trobin <trobin@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef PARSER95_H
#define PARSER95_H

#include "parser.h"
#include "parser9x.h"
#include "word97_generated.h"
#include "word95_generated.h"

namespace wvWare
{

class StyleSheet;
class TextConverter;
class FontCollection;
class ListInfoProvider;
template<class T> class PLCF;

/**
 * This class is the main parser class for Word95 documents.
 */
class Parser95 : public Parser, private Parser9x
{
public:
    Parser95( OLEStorage* storage, OLEStreamReader* wordDocument );
    virtual ~Parser95();

    /**
     * The main parsing method
     */
    virtual bool parse();

    virtual const Word97::FIB& fib() const { return m_fib97; }
    virtual const Word97::DOP& dop() const { return m_dop97; }
    virtual const Word97::FFN& font( S16 ftc ) const;
    virtual AssociatedStrings associatedStrings();
    virtual const StyleSheet& styleSheet() const;

private:
    // don't copy or assing us
    Parser95( const Parser95& rhs );
    Parser95& operator=( const Parser95& rhs );

    void init();
    bool readPieceTable();
    bool parseBody();

    OLEStreamReader* m_data;         // data stream ('Data')

    StyleSheet* m_stylesheet;
    FontCollection* m_fonts;
    ListInfoProvider* m_lists;

    const Word95::FIB m_fib;
    const Word97::FIB m_fib97;
    Word95::DOP m_dop;
    Word97::DOP m_dop97;
    PLCF<Word95::PCD>* m_plcfpcd;     // piece table
    PLCF<Word95::SED>* m_plcfsed;    // section descr. table
    PLCF<Word95::BTE>* m_plcfbtePapx; // PAPX FKP page numbers
    PLCF<Word95::BTE>* m_plcfbteChpx; // CHPX FKP page numbers

    // Some variables we need to hold the current state
    U32 m_currentCP;  // our current position in the document (logical CP)

    TextConverter* m_textconverter;
};

} // namespace wvWare

#endif // PARSER95_H
